# -*- coding: utf-8 -*-

# In[] settings and models
use_gpu_p = True # Set True to use GPU

# model_name_or_path = "TheBloke/Llama-2-13B-chat-GGML"
# model_basename = "llama-2-13b-chat.ggmlv3.q5_1.bin"

# model_name_or_path = "TheBloke/Llama-2-70B-GGML" 
# model_basename = "llama-2-70b.ggmlv3.q8_0.zip"

n_ctx = 8192 # 8192 16384
n_threads = 4

# A txt file that holds the Biobricks name, split by newline
biobricks_txt_file_path = "./biobricks-2410.txt"

# Where the description of Biobricks holds, name + .txt
biobricks_path = "../biobricks-downloads/res"

# Where the summary files saves
res_save_path = "./llm_res/"


# from huggingface_hub import hf_hub_download
# model_path = hf_hub_download(repo_id=model_name_or_path, filename=model_basename)

# model_path = 'llm_models/llama-2-13b-chat.ggmlv3.q5_1.bin' #todo 指定本地路径
model_path = 'llm_models/llama-2-7b-chat.ggmlv3.q5_1.bin'
# model_path = 'llm_models/llama-2-70b.Q3_K_S_2.gguf'

# In[] Test
# Simple test for llama.cpp to generate a BioBrick description
from llama_cpp import Llama 

llm = None

if use_gpu_p:
	# GPU
	llm = Llama(
			model_path=model_path,
			n_threads=n_threads,
			n_batch=512,
			n_gpu_layers=32, #* 最大43
			n_ctx=n_ctx
	)
else:
	# CPU
	llm = Llama(
			model_path=model_path,
			n_threads=n_threads,
			n_ctx=n_ctx
	)

# In[] 
# Load BioBrick Data
biobricks = None

with open(biobricks_txt_file_path) as f:
	biobricks = [line.strip() for line in f.readlines()]
print(biobricks[0:5]) # Take a few as example

def read_biobricks_description(name):
	with open(biobricks_path + name + ".txt") as f:
		return "".join(["" if line.strip() == "" else line.strip() + "\n" for line in f.readlines()])

# print(read_biobricks_description("BBa_B0012")) # Take for an example

# In[]
def llm_generation(description):
	prompt_template1 = '''## Title: Summarize Biobrick Function and Usage
## Instructions:
I will provide you with the Description for a Biobrick part page from the iGEM registry. Please read through the page and summarize the main biological function and potential uses of the part in 1-2 concise sentences. Focus on the key genetic components, mechanisms, and applications but do not include extensive technical details. The summary should help to search out the BioBrick from database.

## Example Description:
medh2 (NAD+ dependent methanol dehydrogenase)
medh2 is an NAD-dependent dehydrogenase catalyzing the following reaction:
CH3OH + NAD(+) → CH2O + NADH
The NAD+ dependent methanol dehydrogenase is encoded by the gene medh2 from Bacillus methanolicus .
The enzyme can use the E. coli's native NAD(+) resources to catalyze the reaction.
The medh gene is cloned from Bacillus methanolicus genomic DNA and the provided sequence is not codonoptimized for protein expression in E. coli .
MEDH2 was successfully expressed in E. coli.
medh2 has a higher in vivo activity in E. coli compared to other methanol dehydrogenases encoded in the Bacillus methanolicus genome (Müller et al.,2015).

## Example Summary(2 or 3 sentences without line feed):
An enzyme that catalyzes the oxidation of methanol to formaldehyde using the cofactor NAD+. Can be expressed in E. coli and utilize native NAD+ resources. Useful for metabolic engineering applications needing conversion of methanol to formaldehyde.

## Description:
''' + description + '''
## Summary:
'''


# 	prompt_template2 = '''## Title: Summarize Biobrick Function and Usage

# ## Instructions:
# I'll provide you the page that describes a Biobrick component. Please read the entire page and write one or two short statements that succinctly describe the basic biological function and potential applications of the component. Concentrate on the essential genetic elements, processes, and applications; avoid going into great technical detail. The summary ought to make it easier to find the BioBrick in the database.


# ## Example Description:

# Terminator (artificial, large, %T~>90)
# Secondary Structure
# File:Mfold-B1006-1.png
# Measurement

# BBa_B1006 parameters Burden Imposed by this Part: Burden Value: 4.3 ± 4.4% Burden is the percent reduction in the growth rate of E. coli cells transformed with a plasmid containing this BioBrick (± values are 95% confidence limits). This BioBrick did not exhibit a burden that was significantly greater than zero (i.e., it appears to have little to no impact on growth). Therefore, users can depend on this part to remain stable for many bacterial cell divisions and in large culture volumes. Refer to any one of the BBa_K3174002 - BBa_K3174007 pages for more information on the methods, an explanation of the sources of burden, and other conclusions from a large-scale measurement project conducted by the 2019 Austin_UTexas team. This functional parameter was added by the 2020 Austin_UTexas team.
# Burden is the percent reduction in the growth rate of E. coli cells transformed with a plasmid containing this BioBrick (± values are 95% confidence limits). This BioBrick did not exhibit a burden that was significantly greater than zero (i.e., it appears to have little to no impact on growth). Therefore, users can depend on this part to remain stable for many bacterial cell divisions and in large culture volumes. Refer to any one of the BBa_K3174002 - BBa_K3174007 pages for more information on the methods, an explanation of the sources of burden, and other conclusions from a large-scale measurement project conducted by the 2019 Austin_UTexas team.
# This functional parameter was added by the 2020 Austin_UTexas team.
# Sequence and Features

# Assembly Compatibility: 10COMPATIBLE WITH RFC[10] 12COMPATIBLE WITH RFC[12] 21COMPATIBLE WITH RFC[21] 23COMPATIBLE WITH RFC[23] 25COMPATIBLE WITH RFC[25] 1000COMPATIBLE WITH RFC[1000]


# ## Example Summary:
# A large, artificial terminator with a high stability value that can be used in E. coli metabolic engineering applications without significantly impacting growth rate or burdening the host cell.


# ## Description:
# ''' + description + '''

# ## Summary:

# '''


	return llm(
		prompt=prompt_template1,
		# prompt=prompt_template2,
		max_tokens=512,
		temperature=0.5,
		top_p=0.95,
		repeat_penalty=1.2,
		top_k=50,
		echo=True,
	)["choices"][0]["text"].split("## Summary:\n", 1)[-1]

# print(llm_generation(read_biobricks_description("BBa_B0012"))) # For example

# In[]
import os

def llm_start(name):
	print(name)
	save_path = res_save_path + name
	if os.path.exists(save_path):
		with open(save_path, 'r') as f:
			readed = f.read()
			return readed
	else:
		try:
			summary = llm_generation(read_biobricks_description(name))
		except Exception as e:
			print(name + " has ERROR")
			print(e)
			return False

		# to_write = '{ "biobrick" : "' + name + '", ' + '"summary" : "' + summary + '"}'
		to_write = '{ "word" : "' + name + '", ' + '"definitions" : "' + summary + '"}'
		with open(save_path, 'w') as f:
			f.write(to_write)
		print(to_write)
		return to_write

# In[]
data = [llm_start(name) for name in biobricks]
