"""



"""
from fastNLP.io import  DataBundle, Pipe
from fastNLP import DataSet
# from transformers.tokenization_bert import BertTokenizer 
from transformers.models.bert.tokenization_bert import BertTokenizer # from transformers.models.bert.tokenization_bert import BasicTokenizer
from .loader import CnLoader, EnLoader, BioLoader
from transformers import RobertaTokenizer


def _prepare_data_bundle(tokenizer, data_bundle, max_word_len):
	# 下面是几个常用标记对应的id
	mask_id = tokenizer.convert_tokens_to_ids(['[MASK]'])[0] # 遮盖标记
	sep_id = tokenizer.convert_tokens_to_ids(['[SEP]'])[0] # 分隔标记
	cls_id = tokenizer.convert_tokens_to_ids(['[CLS]'])[0] # 表示整个输入序列的开始
	pad_id = tokenizer.convert_tokens_to_ids(['[PAD]'])[0] # 填充序列，以便使它们具有相同的长度
	unk_id = tokenizer.convert_tokens_to_ids(['[UNK]'])[0] # 表示未知或未登录的词汇
	# 当模型在预训练或推理阶段遇到不在词汇表中的词汇时，通常会将它们替换为 [UNK] 标记，以处理模型未见过的词汇。

	target_words = data_bundle.target_words
	word2bpes = [] # word对应的PBE编码
	word2idx = {} # word对应的index

	# 下面这两个for是完成
	for ins in data_bundle.get_dataset('train'): # 遍历词条
		word = ins['word']
		if word not in word2idx:
			word2idx[word] = len(word2idx)
			bpes = tokenizer.convert_tokens_to_ids(tokenizer.tokenize(word))
			word2bpes.append(bpes)
	number_word_in_train = len(word2idx)
	for word in target_words: 
		if word not in word2idx:
			word2idx[word] = len(word2idx)
			bpes = tokenizer.convert_tokens_to_ids(tokenizer.tokenize(word))
			word2bpes.append(bpes)
	
	for name in data_bundle.get_dataset_names(): # 遍历数据集 name: train, seen, unseen, dev
		# (1) 需要tokenize word列
		# (2) 需要tokenize definition列
		ds = data_bundle.get_dataset(name) # 根据name获得对应数据集
		new_ds = DataSet() #* 创建新的数据集
		# 如果为dict类型，则每个key的value应该为等长的list; 如果为list，每个元素应该为具有相同field的 
		
		for ins in ds: # 遍历数据集中的每个词条
			word = ins['word'] # 获得待查word
			idx = word2idx[word] # 获得待查word的index
			bpes = word2bpes[idx] # 获得待查word对应的bpe编码
			definition = [] #
			
			# .分词. 将每个词语转为BPE编码后存入definition
			for word in ins['definition'].split(): # ins['definition'].split()对ins词条中的definition用空格拆分成一个的词, 然后遍历
				# tokenizer.tokenize(word)：使用给定的 tokenizer 对每个词汇进行分词操作，将其转换为 BPE 编码序列。
				definition.extend(tokenizer.tokenize(word)) # 将分词后的 BPE 编码序列逐一添加到 definition 列表中。

			# tokenizer 是一个预训练的 tokenizer 对象，它包含了将文本转换为 BPE 编码和将 BPE 编码转换为整数 ID 的功能。通常，这个 tokenizer 是针对特定的预训练模型（如 BERT、RoBERTa 等）训练的，可以确保文本和编码之间的一致性。
			# convert_tokens_to_ids(definition): 这是 tokenizer 对象的一个方法，用于将 BPE 编码的词汇序列 definition 转换为整数 ID 序列。
				# 具体来说，它会遍历输入的 BPE 编码词汇列表，查找每个词汇在*预训练模型的词汇表中*对应的整数 ID，并将这些整数 ID 组成一个新的序列,返回到definition中。
			# print( 'before change: ', definition) 
			# .字母转编码. 对于预训练模型库中没有的单词,会被拆散后编码
			definition = tokenizer.convert_tokens_to_ids(definition)
			# print( 'after change: ',definition ) 

			# 创建用于模型输入的序列 input，归一化后将其存储在词条 ins 中的 'input' 字段中
			input = [cls_id] + [mask_id] * max_word_len + [sep_id] + definition
			# 在 BERT 和类似的模型中，[CLS] 标记用于表示整个序列的开始; mask 表示模型在预训练阶段用于掩盖词汇的标记,将 [mask_id] 重复 max_word_len 次，以确保掩盖足够长度的词汇;
			# [SEP] 标记用于表示序列的分隔。definition是单词分词后转成的编码
			input = input[:256] # 最大长度为256; 不是此处的问题
			# input = input[:512] # 最大长度为256; 不是此处的问题
			input.append(sep_id) # 向input序列的末尾添加一个 [SEP] 标记, 表示序列的结束
			ins['input'] = input # 存储在词条 ins 中的 'input' 字段中
			
			if unk_id in bpes: # 遍历word2bpes所有bpes发现,没有bpes是没编码的, 调试也验证, So此if下面几行不会执行
				if name == 'train':
					continue
				else: # 如果目标词汇的 BPE 编码中包含 [UNK]，则将其编码设为全零序列。这是为了处理测试集中可能包含未知词汇的情况。
					bpes = [0] * (max_word_len + 1)  # 使得设置target为-1
			
			if len(bpes) <= max_word_len: #**! max_word_len要够大!
				ins['target'] = idx # 
				new_ds.append(ins)
			else:
				if name != 'train':
					ins['target'] = -1
					new_ds.append(ins)
		data_bundle.set_dataset(new_ds, name)
	
	# 中文就不考虑start和middle bpe的区别了

	for i in range(len(word2bpes)): #* 将word2bpes归一化为max_word_len长度, 不够的用mask_id填充
		bpes = word2bpes[i]
		bpes = bpes[:max_word_len] + [mask_id] * max(0, max_word_len - len(bpes))
		word2bpes[i] = bpes

	data_bundle.set_pad_val('input', pad_id)
	data_bundle.set_input('input')
	data_bundle.set_target('target')
	setattr(data_bundle, 'word2bpes', word2bpes)
	setattr(data_bundle, 'pad_id', pad_id)
	setattr(data_bundle, 'number_word_in_train', number_word_in_train)
	setattr(data_bundle, 'word2idx', word2idx)

	return data_bundle



class ENBertPipe(Pipe):
	"""
	输出为

		target          input
		int             [cls, xx]
	并包含两个attr
		first_word2bpe: 出现在首位的bpe
		middle_word2bpe: 出现在中间的bpe

	"""
	def __init__(self, bert_name, max_word_len=6):
		super().__init__()
		self.bert_name = bert_name
		self.max_word_len = max_word_len
		self.lower = True

	def process(self, data_bundle):
		"""
		输入为
			word                definition
			测试                  这是 一个 测试

		:param data_bundle:
		:return:
		"""
		# tokenizer = BertTokenizer.from_pretrained(self.bert_name) #** 
		#** 创建英文tokenizer
		tokenizer = BertTokenizer.from_pretrained('pretrained_models/bert-base-uncased') #** 

		tokenizer.do_basic_tokenize = False #* 不是这的问题

		return _prepare_data_bundle(tokenizer, data_bundle, self.max_word_len)

	def process_from_file(self, paths) -> DataBundle:
		# data_bundle = EnLoader().load(paths) #** 加载数据, 这的data_bundle没问题
		data_bundle = BioLoader().load(paths)
		return self.process(data_bundle) 


class CNBertPipe(Pipe): #* 创建中文tokenizer
	"""
	由于中文roberta使用的是hit的，所以直接用这个bert load就好了
	"""
	def __init__(self, bert_name, max_word_len=6): #** mono/train_cn_bert.py中的bert_name=hfl/bert-chinese-wwm
		super().__init__()
		self.bert_name = bert_name
		self.max_word_len = max_word_len

	def process(self, data_bundle):
		"""
		输入为
			word                definition
			测试                  这是 一个 测试

		:param data_bundle:
		:return:
		"""
		# tokenizer = BertTokenizer.from_pretrained( self.bert_name) #** 
		#* 创建中文tokenizer所需要的配置文件, 这是加载数据前需要创建的文件
		tokenizer = BertTokenizer.from_pretrained( 
			'pretrained_models/hfl--chinese-bert-wwm', 
			# 'pretrained_models/hfl--chinese-bert-wwm-ext', 
			# local_files_only=True, Whether or not to only rely on local files and not to attempt to download any files #* 此参数不需要; 
			) 
		
		# 本地可翻墙时用token可下载
		# tokenizer = BertTokenizer.from_pretrained( pretrained_model_name_or_path=self.bert_name, token='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' ) 
		tokenizer.do_basic_tokenize = True
		return _prepare_data_bundle(tokenizer, data_bundle, self.max_word_len)

	def process_from_file(self, paths) -> DataBundle:
		data_bundle = CnLoader().load(paths) #** 加载数据函数
		return self.process(data_bundle) 


class ENRobertaPipe(Pipe):
	def __init__(self, roberta_name, max_word_len):
		super().__init__()
		self.roberta_name = roberta_name
		self.max_word_len = max_word_len

	def process(self, data_bundle):
		tokenizer = RobertaTokenizer.from_pretrained(self.roberta_name)
		mask_id = tokenizer.convert_tokens_to_ids(['<mask>'])[0]
		sep_id = tokenizer.convert_tokens_to_ids(['</s>'])[0]
		cls_id = tokenizer.convert_tokens_to_ids(['<s>'])[0]
		pad_id = tokenizer.convert_tokens_to_ids(['<pad>'])[0]
		unk_id = tokenizer.convert_tokens_to_ids(['<unk>'])[0]

		target_words = data_bundle.target_words
		word2bpes = []
		word2idx = {}
		for ins in data_bundle.get_dataset('train'):
			word = ins['word']
			if word not in word2idx:
				word2idx[word] = len(word2idx)
				bpes = tokenizer.convert_tokens_to_ids(tokenizer.tokenize(' ' + word))
				word2bpes.append(bpes)

		number_word_in_train = len(word2idx)

		for word in target_words:
			if word not in word2idx:
				word2idx[word] = len(word2idx)
				bpes = tokenizer.convert_tokens_to_ids(tokenizer.tokenize(' ' + word))
				word2bpes.append(bpes)

		for name in data_bundle.get_dataset_names():
			# (1) 需要tokenize word列
			# (2) 需要tokenize definition列
			ds = data_bundle.get_dataset(name)
			new_ds = DataSet()
			for ins in ds:
				word = ins['word']
				idx = word2idx[word]
				bpes = word2bpes[idx]
				definition = ins['definition'].split()
				words = []
				for word in definition:
					word = tokenizer.tokenize(' ' + word)
					word = tokenizer.convert_tokens_to_ids(word)
					words.extend(word)
				input = [cls_id] + [mask_id] * self.max_word_len + \
						[sep_id]*2 + words
				input = input[:256]
				input.append(sep_id)
				ins['input'] = input
				if unk_id in bpes:
					if name == 'train':
						continue
					else:
						bpes = [0] * (self.max_word_len+1)  # 使得设置target为-1
				if len(bpes) <= self.max_word_len:
					ins['target'] = idx
					new_ds.append(ins)
				else:
					if name != 'train':
						ins['target'] = -1
						new_ds.append(ins)
			data_bundle.set_dataset(new_ds, name)

		for i in range(len(word2bpes)):
			bpes = word2bpes[i]
			bpes = bpes[:self.max_word_len] + [mask_id]*max(0, self.max_word_len-len(bpes))
			word2bpes[i] = bpes

		data_bundle.set_pad_val('input', pad_id)
		data_bundle.set_input('input')
		data_bundle.set_target('target')
		setattr(data_bundle, 'word2bpes', word2bpes)
		setattr(data_bundle, 'pad_id', pad_id)
		setattr(data_bundle, 'number_word_in_train', number_word_in_train)
		setattr(data_bundle, 'word2idx', word2idx)
		return data_bundle

	def process_from_file(self, paths) -> DataBundle:
		data_bundle = EnLoader().load(paths)
		return self.process(data_bundle)


# if __name__ == '__main__': #?** 这个作用是?
# 	path = '../../../data/cn'
# 	pipe = CNBertPipe('cn')
# 	data_bundle = pipe.process_from_file(path)
# 	print(data_bundle)

# if __name__ == '__main__':
# 	path = '../../../data/cn'
# 	pipe = CNBertPipe('cn')
# 	data_bundle = pipe.process_from_file(path)
# 	print(data_bundle)
