# Introduction

The `mono/` directory contains these files

* `train_en_bert.py`: Use the summary data and the pre-trained model to train the BERT model.
* `use_bio_cmd.py`: Run model inference in the terminal
* `web_ui.py`: A simple web interface to call the trained model for inference for users to use

# Programming Guide

## Environment Configuration

### Model Training

We recommend creating a new environment using `conda create -n xxx python==3.10` (optional)

* torch2.0.1: `pip3 install torch torchvision torchaudio --index-url https://download.pytorch.org/whl/cu118`
  (Install PyTorch according to your version of Python and CUDA; See [pytorch.org](https://pytorch.org/get-started/locally/) for more information)
* `pip install nltk`
* `pip install fastNLP==0.6.0/0.7.0` (See [fastNLP](https://github.com/fastnlp/fastNLP); You need to download the specified old version to run this code.)
* `pip install transformers`
* `pip install fitlog`
  (If an error occurred in ``.../fitlog/fastlog/logger.py, line 776``, you may need to revise `np.str` to `np.str_`)

### Website Building

* Install Gradio: `pip install gradio`
* A host with GPU

## Prepare the dataset

The dataset we used is in [data/biobricks](../data/biobricks/), We need these json files to train and test our model:

* train.json: train data
* seen.json: test data which is selected from train data
* unseen.json: test data which train data do not have
* dev.json: the merge of seen and unseen

you could see that all the json files have the same format: 

```
[
	{
		"word": "<the word you want to search>",
		"definitions": "<the discription of this word>"
	},
	{
		"word": "...",
		"definitions": "..."
	},
	...
	{
		"word": "...",
		"definitions": "..."
	}
]
```

So you need to change your data into this kind of format to match the code.

NOTE: the test dataset can have more, you can just change the code in mono/data/load.py to match your data.

## Download the Models

The English version: [bert-base-uncased at main (huggingface.co)](https://huggingface.co/bert-base-uncased/tree/main)

- Download `pytorch.bin` into `mono\pretrained_models\bert-base-uncased`

The Chinese version: [hfl/chinese-bert-wwm at main (huggingface.co)](https://huggingface.co/hfl/chinese-bert-wwm/tree/main)

- Download `pytorch.bin` into `mono\pretrained_models\hfl--chinese-bert-wwm`

if you want use other new models on [huggingface.co](https://huggingface.co/bert-base-uncased/tree/main), that would be ok; And you need to down similar files in `mono\pretrained_models\bert-base-uncased `on your aim repository on huggingface and put them into a new folder.

## Running Models

### Workflow

- Load data: According to the `pre_name` specified in the `train_xxx.py`, see whether data files have been specified in `mono/caches`. If files exist, load the cache files; otherwise, generate a tokenizer according to the filepath specified in `mono/data/pipe.py`, and load data according to the `paths='../data/biobricks'`. Therefore, pre_name needs to be changed when another dataset is used.
- Load Model: Load the pre-trained model according to the filepath specified in `mono/model/bert.py`
- Start Training

### Parameters

These parameters need to be modified:

- mono/train_xxx.py：pre_name, device（specify the gpu device）
- mono/data/pipe.py: path
- mono/model/bert.py: path for the trained model
- ......

# Reference

* [https://github.com/yhcc/BertForRD](https://github.com/yhcc/BertForRD)
* [https://github.com/thunlp/MultiRD](https://github.com/thunlp/MultiRD)
