
'''
该程序文件是一个用于训练中文BERT模型的脚本。它使用了fastNLP库来进行训练，并使用了预训练的BERT模型。
该脚本的主要功能包括数据处理、模型构建、优化器定义、训练和评估等。
它使用了一些回调函数来控制训练过程中的一些操作，如梯度裁剪、学习率预热等。训练结束后，模型会保存在指定的路径下。
'''

import sys
sys.path.append('../') #* 以mono为工作区, 则要加入../, 
#* 上面这个是对 from mono.data.pipe import CNBertPipe 等import语句有用, 对其他的如paths = '../data/cn'无影响! 
#** 注意, mono文件夹下代码需要在以mono为工作区文件夹下的命令行里运行!!, 自定义相对路径时要以mono为工作区! 
# sys.path.append('./') # 工作区/运行命令路径为BertForRD文件夹,加入./这个, 或者不用加 
# sys.path.append('/data1/jianshu_zeng/iGEM_RD/BertForRD/')

import os
if 'p' in os.environ:
	os.environ['CUDA_VISIBLE_DEVICES']=os.environ['p']

import torch
from torch import optim
from fastNLP import Trainer, CrossEntropyLoss
from fastNLP import BucketSampler, cache_results, WarmupCallback, GradientClipCallback, FitlogCallback
from mono.data.pipe import CNBertPipe
from mono.model.bert import CNBertReverseDict
import fitlog
from mono.model.metrics import MonoMetric
from joint.data.utils import clip_max_length
# fitlog.debug()
fitlog.set_log_dir('cn_logs')
fitlog.add_hyper_in_file(__file__)


paths = '../data/cn' #* 
#########hyper
model_name = 'bert'  # change this to roberta, you can run the roberta version
max_word_len = 4
lr = 5e-5
batch_size = 64
n_epochs = 15
#########hyper
if model_name == 'bert':
	# pre_name = 'hfl/bert-chinese-wwm' # 'hfl/chinese-bert-wwm-ext'
	pre_name = 'chinese-bert-wwm'
	# pre_name = 'chinese-bert-wwm-ext' #** 
	
elif model_name=='roberta':
	pre_name = 'chinese_roberta_wwm_ext'
else:
	raise Exception()

#* caches文件存储/读取路径
@cache_results('caches/cn_{}.pkl'.format(pre_name.split('/')[-1], max_word_len), _refresh=False)
def get_data():
	data_bundle = CNBertPipe(pre_name, max_word_len).process_from_file(paths) #**
	return data_bundle

data_bundle = get_data()
print(data_bundle)
word2bpes = data_bundle.word2bpes
print(f"In total {len(word2bpes)} target words")
pad_id = data_bundle.pad_id

model = CNBertReverseDict(pre_name, word2bpes, pad_id=pad_id,  #*** 加载预训练模型
						number_word_in_train=data_bundle.number_word_in_train)

if torch.cuda.is_available():
	model.cuda()

optimizer = optim.AdamW(model.parameters(), lr=lr)

data = {}
for name in ['desc', 'question', 'seen_test', 'unseen_test']: #* 测试数据
	data[name] = data_bundle.get_dataset(name)

callbacks = [GradientClipCallback(clip_type='value'), WarmupCallback(warmup=0.1, schedule='linear')]
callbacks.append(FitlogCallback(data=data, verbose=1))

train_data = data_bundle.get_dataset('train') #* 训练数据
train_data.add_seq_len('input')
sampler = BucketSampler()

clip_max_length(train_data, data_bundle)

trainer = Trainer(train_data=train_data, model=model,
				optimizer=optimizer, loss=CrossEntropyLoss(),
				batch_size=64, # 80
				sampler=sampler, drop_last=False, update_every=1,
				num_workers=1, print_every=5,
				n_epochs=20, # 20
				dev_data=data_bundle.get_dataset('dev'), 
				metrics=MonoMetric(),
				metric_key='t1',
				validate_every=-1, save_path='save_models/', use_tqdm=True, 
				# device=None,
				device=6, #** 指定gpu设备,10服务器选0-6
				callbacks=callbacks, check_code_level=0)

trainer.train(load_best_model=False)

fitlog.add_other(trainer.start_time, name='start_time')

