import traceback
import numpy as np
import torch

# torch.set_printoptions(threshold=np.inf)
from mono.model.bert import ENBertReverseDict
from transformers import BertTokenizer
from tqdm import tqdm

import sys

sys.path.append("../")

word2bpes = np.load("./bio_word2bpes.npy").tolist()  # 需为list
target_words = np.load("./bio_target_words.npy")
N = 2410

# ** 加载预训练模型
# print( '开始加载模型: ' )
model = ENBertReverseDict(word2bpes=word2bpes, pad_id=0, number_word_in_train=N)
states = torch.load("save_models/bio-rd-t1-0.bin").state_dict()
model.load_state_dict(states)
# print(model)

model.eval()
if torch.cuda.is_available():
    model.cuda(0)
# print( '模型加载完毕; ' )


# input: 用户输入的str
def input_to_bpe(input_str):
    # max_word_len = 4
    tokenizer = BertTokenizer.from_pretrained(
        "pretrained_models/bert-base-uncased"
    )  # **
    tokenizer.do_basic_tokenize = True
    mask_id, sep_id, cls_id, pad_id, unk_id = 103, 102, 101, 0, 100

    out_bpe = []
    for word in input_str.split():  # 用空格拆分成一个个的词, 然后遍历
        out_bpe.extend(tokenizer.tokenize(word))  # 将分词后的 BPE 编码序列逐一添加到 definition 列表中。
    out_bpe = tokenizer.convert_tokens_to_ids(out_bpe)

    out_matrix = []

    for i in tqdm(range(N)):
        # output = [cls_id] + [mask_id] * max_word_len  + [sep_id] + out_bpe
        out_list = [cls_id] + word2bpes[i] + [sep_id] + out_bpe
        # out_list = [cls_id] + word2bpes[0] + [sep_id] + out_bpe
        out_list = out_list[:256]
        out_list.append(sep_id)
        out_matrix.append(out_list)
    # return out_bpe
    return out_matrix  # 2410行*2410列矩阵


# 输入字符串; 返回
def run(word):
    try:
        while True:
            # word = input()
            bpe_matrix = input_to_bpe(word)
            batch_size = 128
            res = []
            # print( '开始推理:' )
            for i in tqdm(range(N // batch_size + 1)):  # 2410/64=38, 2410/32=76
                tensor_bpe = torch.tensor(
                    bpe_matrix[i * batch_size : min((i + 1) * batch_size, N)]
                ).cuda(
                    0
                )  # *  batch_size行2410列

                with torch.no_grad():  # 若不加此行, 直接写 res_dict = model( bpe ) 或res_dict = model.forward( bpe ) 输出格式不太对
                    res_dict = model(tensor_bpe)
                    # res_dict = model.forward( tensor_bpe )
                # print(res_dict)
                res_matrix = res_dict["pred"].cpu().numpy()  # batch_size行2410列的二维数组

                for j in range(len(res_matrix)):
                    res.append(
                        res_matrix[j][i * batch_size + j]
                    )  # * 获取大矩阵左上右下对角线上的元素,表示该词条和target[i]的相关性; 一般就是该行最大值

            res = np.array(res)
            top_num = res.argsort()[-10:][::-1]  # 获取numpy数组中的10个最大值的索引(从大往小排列)
            # top_num = np.array(res).argsort()[-10:][::-1]
            # print( res[top_num] ) #* numpy数组才能这样索引
            # print( top_num )
            # print( target_words[top_num] )

            # 返回三个数组: 前x个目标的位置, 预测分数, 和对应的词
            return (top_num, res[top_num], target_words[top_num])

    except Exception as e:
        print(e)
        traceback.print_exc()
