import gradio as gr
import json


import sys
import os
from use_bio_html import run


HTML_BASE = """<div style="padding-left: 5px; padding-right: 0px;">
<style>

</style>
<div style="font-size:20px;display:flex;">
	<strong>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Name&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</strong>	
	<strong style="word-wrap:break-word;padding: 0px 50px 25px 60px;text-align: center;">Definition</strong>
</div>
<ol start="1" style="color:grey">
"""

HTML_TEMPLATE = """
<li style="font-size:20px;display:flex;" class="textspan svelte-ju12zg hl ">
	<a href="http://parts.igem.org/Part:{name}" target="_blank" style="width:128px;"><u>{name}</u></a>	
	<div style="word-wrap:break-word;padding: 0px 50px 25px 50px">{definition}</div>
</li>
"""


with open("target_words.txt") as f:
    word_list = f.readlines()


with open("desc_pre.json") as f:
    word_desriptions = json.load(f)


def calcolor(p):
    if p > 0.9:
        return "#005aff64"
    if p < 0.1:
        return "#005aff00"
    return "#005aff" + "{0:02d}".format(int(p * 80 - 8))  # 0.1->00,0.9->64,linearly


def predict(text):
    res_index, res_score, res_name = run(text)

    html_res = (
        HTML_BASE
        + "".join(
            [
                HTML_TEMPLATE.format(
                    color=calcolor(1 - i / len(res_index)),
                    definition=word_desriptions[word_list[res_index[i]].strip("\n")],
                    name=word_list[
                        res_index[i]
                    ],  # .ljust(13,'\n').replace('\n','&nbsp;')
                )
                for i in range(len(res_index))
            ]
        )
        + """</ol></div>"""
    )
    return html_res


with gr.Blocks(title="Ask NOX") as demo:
    gr.Markdown("# Welcome to use Ask NOX!")
    input = gr.Textbox(
        label="", placeholder="Please type the description of the biobricks you want..."
    )
    greet_btn = gr.Button("Look up")
    output = gr.HTML()
    greet_btn.click(fn=predict, inputs=input, outputs=output, api_name="predict")

demo.launch()
